#include <cstdlib>
#include <cstdio>
#include <vector>
#include <algorithm>

using namespace std;

//#define debug(fmt, ...) fprintf(stderr, fmt, ## __VA_ARGS__)
#define debug(fmt, ...) 

struct edge {
    int a, b, company;
    inline edge() { }
    inline edge(int _a, int _b, int _company) : a(_a), b(_b), company(_company) { }
    inline bool operator<(const edge &e) const {
        return a != e.a ? a < e.a :
               b != e.b ? b < e.b :
               company < e.company;
    }
};

static int owns[8001][101][2];

struct endpoint_info {
    int x, y, date;
    int next_a, next_b;
};
static endpoint_info endpoints[8001][101];
static int epi_date_gen;
#define EPI_STEP 90

/* -------------------------------------------------------------------------- */

static int v_path_next(int com, int a, int b) {
    return owns[a][com][0] == b ? owns[a][com][1] : owns[b][com][0];
}
static int ep_path_next(int com, int a, int b) {
    return endpoints[a][com].next_a == a ? endpoints[a][com].next_b : endpoints[a][com].next_a;
}

static vector<int> v_get_path(int com, int a, int b, int len) {
    vector<int> ret;
    ret.push_back(a);
    while(--len) {
        int next = v_path_next(com,a,b);
        if(next == -1) break;
        b = a;
        a = next;
        ret.push_back(a);
    }
    return ret;
}
static vector<int> ep_get_path(int com, int a, int b, int len) {
    vector<int> ret;
    ret.push_back(a);
    while(--len) {
        int next = ep_path_next(com,a,b);
        if(next == -1) break;
        b = a;
        a = next;
        ret.push_back(a);
    }
    return ret;
}

static endpoint_info get_epi_dir(int com, int a, int b)
static endpoint_info get_epi(int com, int a)
{
    endpoint_info ret = endpoints[a][com];
    if(owns[a][com][0] != -1) ret = max(ret, get_epi_dir(com, a, owns[a][com][0]))
}

static void link_paths(int com, int a, int b)
{
    vector<int> Pa = v_get_path(com, a, b, EPI_STEP),
                Pb = v_get_path(com, b, a, EPI_STEP);

    endpoint_info EPIa = get_epi(com, a),
                  EPIb = get_epi(com, b),
                  EPI;
    
    if(EPIa.x == a) EPI.x = EPIa.y;
    else if(EPIa.y == a) EPI.x = EPIa.x;
    else assert(false);
    
    if(EPIb.x == b) EPI.y = EPIb.y;
    else if(EPIb.y == b) EPI.y = EPIb.x;
    else assert(false);

    EPI.date = ++epi_date_gen;

    for(int i=0; i<Pa.size(); i++) {
        int j = EPI_STEP - i - 1;
        if(j >= Pb.size() || j < 0) continue;

        int v = Pa[i], u = Pb[j];
        if(endpoints[v].next_a == -1) endpoints[v].next_a = u;
        else if(endpoints[v].next_b == -1) endpoints[v].next_b = u;
        else assert(false);
        
        if(endpoints[u].next_a == -1) endpoints[u].next_a = v;
        else if(endpoints[u].next_b == -1) endpoints[u].next_b = v;
        else assert(false);
    }

    vector<int> Qa = ep_get_path(com, a, b, EPI_STEP),
                Qb = ep_get_path(com, b, a, EPI_STEP);

    for(int i=0; i<Qa.size(); i++) {
        endpoints[Qa[i]].x = EPI.x;
        endpoints[Qa[i]].y = EPI.y;
        endpoints[Qa[i]].date = EPI.date;
    }
    for(int i=0; i<Qb.size(); i++) {
        endpoints[Qb[i]].x = EPI.x;
        endpoints[Qb[i]].y = EPI.y;
        endpoints[Qb[i]].date = EPI.date;
    }
}

/* -------------------------------------------------------------------------- */


static int cnt_own(int srv, int com) {
    int ret = (owns[srv][com][0] != -1) + (owns[srv][com][1] != -1);
    debug("company %d has %d cables at server %d\n", com, ret, srv);
    return ret;
}
static bool add_own(int srv, int com, int eidx) {
    if(owns[srv][com][0] == -1)
        owns[srv][com][0] = eidx;
    else if(owns[srv][com][1] == -1)
        owns[srv][com][1] = eidx;
    else {
        debug("ERROR: can't add cable %d of company %d to server %d\n", eidx,com,srv);
        return false;
    }
    debug("added cable %d of company %d to server %d\n", eidx,com,srv);
    return true;
}
static bool del_own(int srv, int com, int eidx) {
    if(owns[srv][com][0] == eidx) { owns[srv][com][0] = -1; debug("removed cable %d of company %d from server %d\n", eidx,com,srv); return true; }
    if(owns[srv][com][1] == eidx) { owns[srv][com][1] = -1; debug("removed cable %d of company %d from server %d\n", eidx,com,srv); return true; }
    debug("ERROR: can't delete cable %d of company %d to server %d\n", eidx,com,srv);
    return false;
}

static void solve()
{
    int n,m,c,t;
    scanf("%d %d %d %d",&n,&m,&c,&t);
    if(!n && !m && !c && !t)
        exit(0);

    for(int i=0; i<=n; i++)
        for(int j=0; j<=c; j++)
            owns[i][j][0] = owns[i][j][1] = -1;

    vector<edge> E(m);
    for(int i=0; i<m; i++) {
        scanf("%d %d %d",&E[i].a,&E[i].b,&E[i].company);
        add_own(E[i].a, E[i].company, i);
        add_own(E[i].b, E[i].company, i);
    }

    sort(E.begin(), E.end());

    while(t--) {
        int a,b,k;
        scanf("%d %d %d",&a,&b,&k);
        debug("trade cable %d <--> %d to company %d\n", a,b,k);

        int idx = lower_bound(E.begin(), E.end(), edge(a,b,-1)) - E.begin();
        if(idx == m || E[idx].a != a || E[idx].b != b) {
            printf("No such cable.\n");
            continue;
        }
        if(E[idx].company == k) {
            printf("Already owned.\n");
            continue;
        }
        if(cnt_own(a, k) == 2 || cnt_own(b, k) == 2) {
            printf("Forbidden: monopoly.\n");
            continue;
        }
        if(cnt_own(a, k) == 1 && cnt_own(b, k) == 1) {
            printf("Forbidden: redundant.\n");
            continue;
        }
        del_own(a,E[idx].company,idx);
        del_own(b,E[idx].company,idx);
        add_own(a,k,idx);
        add_own(b,k,idx);
        E[idx].company = k;
        printf("Sold.\n");
    }

    printf("\n");
}

int main(void) {
	for(;;) solve();
}

